#!/bin/sh

prefix="/usr"
exec_prefix="/usr"
datarootdir="${prefix}/share"

export PATH="${PATH}:/usr/script"

if [ -d "/home/root" ]; then
	export HOME="/home/root"
	cd
fi

BRANDCHECK=`sed -En 's|^brand=(.+)$|\1|p' /usr/lib/enigma.info`
PLATFORMCHECK=`sed -En 's|^platform=(.+)$|\1|p' /usr/lib/enigma.info`
PYVER=`sed -En 's|^python=(.+)$|\1|p' /usr/lib/enigma.info`

if [ -f "/usr/bin/libreader" ]; then
	COUNT=`ps -ef | grep libreader | grep -v "grep" | wc -l`
	if [ "${COUNT}" -eq "0" ];then
		/usr/bin/libreader 720P_50
	fi
fi

if [ -f "/proc/stb/fp/led_color" ]; then
	echo "00ffffff" > /proc/stb/fp/led_color
elif [ -f "/proc/stb/fp/led_brightness" ]; then
	echo "000000ff" > /proc/stb/fp/led_brightness
fi

if [ "${PLATFORMCHECK}" == "dmamlogic" ]; then
	/sbin/insmod /lib/modules/4.9/extra/lcd/lcd.ko
	echo "1" > /proc/stb/fp/nec_ir
	[ -f "/sys/class/remote/amremote/protocol" ] && echo "2" > /sys/class/remote/amremote/protocol
fi

# LC_ALL available?
if [ -z $LC_ALL ] && [ ${PYVER::1} == 3 ]; then
	export LC_ALL=en_GB.utf8
fi

# Check for sundtek tuner helper lib.
if [ -e "/opt/lib/libmediaclient.so" ]; then
	LIBS="/opt/lib/libmediaclient.so /usr/lib/libopen.so.0.0.0"
else
	LIBS="/usr/lib/libopen.so.0.0.0"
fi

DEBUGDIR=`sed -En 's|^config\.crash\.debugPath=(.+)$|\1|p' "/etc/enigma2/settings"`

# Enable and configure gstreamer debug log of category and level without color
if [ "$(grep -i config.crash.gstdebug=true /etc/enigma2/settings)" != "" ]; then
	GSTDEBUGCATEGORY="*"
	GSTDEBUGLEVEL="INFO"
	if [ "$(grep config.crash.gstdebugcategory= /etc/enigma2/settings | sed 's/config.crash.gstdebugcategory=//g')" != "" ]; then
		GSTDEBUGCATEGORY=$(grep config.crash.gstdebugcategory= /etc/enigma2/settings | sed 's/config.crash.gstdebugcategory=//g')
	fi
	if [ "$(grep config.crash.gstdebuglevel= /etc/enigma2/settings | sed 's/config.crash.gstdebuglevel=//g')" != "" ]; then
		GSTDEBUGLEVEL=$(grep config.crash.gstdebuglevel= /etc/enigma2/settings | sed 's/config.crash.gstdebuglevel=//g')
	fi
	export GST_DEBUG_FILE="${DEBUGDIR}$(date +%Y-%m-%d_%H-%M-%S)-gstreamer-debug.log"
	export GST_DEBUG="${GSTDEBUGCATEGORY}:${GSTDEBUGLEVEL}"
	export GST_DEBUG_NO_COLOR="1"
fi

# Enable and set gstreamer directory for pipeline graphs that are not created if directory is not set
if [ "$(grep -i config.crash.gstdot=true /etc/enigma2/settings)" != "" ]; then
	export GST_DEBUG_DUMP_DOT_DIR=${DEBUGDIR}
fi

# Enable generation of core dumps
# It would be best to have this at system startup because this way we may not
# catch core dumps that happen early to enigma2.
# The kernel.core_pattern could be set via /etc/sysctl.conf.
# The size limit could be set via /etc/security/limits.conf.
if [ "$(grep -i config.crash.coredump=true /etc/enigma2/settings)" != "" ]; then
	if [ -n "${DEBUGDIR}" ] && [ -d "${DEBUGDIR}" ] || DEBUGDIR="/home/root/logs/" ]; then
		COREDUMPSIZE=$((10 * 512 * 4 - 1 * 512 * 4)) # size is in blocks (512 byte) so this sets 10 MB - 1 MB for log files
		COREDUMPFILE=${DEBUGDIR}%t-enigma2-core.dump
	fi
	if [ "$(grep -i config.crash.sizeloglimit= /etc/enigma2/settings | sed 's/config.crash.sizeloglimit=//g')" != "" ]; then
		COREDUMPSIZE=$(grep -i config.crash.sizeloglimit= /etc/enigma2/settings | sed 's/config.crash.sizeloglimit=//g')
		COREDUMPSIZE=$((${COREDUMPSIZE} * 512 * 4 - 1 * 512 * 4))
	fi
	echo ${COREDUMPFILE} > /proc/sys/kernel/core_pattern
	ulimit -c ${COREDUMPSIZE}
fi

enigma2pid=$!

# Enable generation of core dumps with ELF header included.
# This seem to be required by gdb.
# Would be default if kernel is configured with CONFIG_CORE_DUMP_DEFAULT_ELF_HEADERS.
# Default coredump filter can only be set via kernel command line.
# In all other cases this is only available per process.
if [ "$(grep -i config.crash.coredump=true /etc/enigma2/settings)" != "" ]; then
	COREDUMPFILTER=0x33
	echo ${COREDUMPFILTER} > /proc/${enigma2pid}/coredump_filter
fi

wait ${enigma2pid}

# This should not be done as it will suppress the appropriate diagnostic message from within Enigma2.
#
# if [ ! -e "/etc/enigma2/settings" ]; then
# 	touch /etc/enigma2/settings
# fi

# Enigma main loop.
while : ; do
	# Show backdrop on enigma2 start.
	killall -9 showiframe; sleep 1
	SKIN=`sed -En 's|^config\.skin\.primary_skin=(.+)/skin\.xml$|\1|p' "/etc/enigma2/settings"`
	if [ -n "${SKIN}" ]; then
		SEARCHDIRS="/etc/enigma2/$SKIN /etc/enigma2/skin_common /etc/enigma2 /usr/share/enigma2/$SKIN /usr/share/enigma2/skin_default /usr/share/enigma2"
	else
		SEARCHDIRS="/etc/enigma2/skin_common /etc/enigma2 /usr/share/enigma2/skin_default /usr/share/enigma2"
	fi
	for DIR in $SEARCHDIRS; do
		if [ -d "${DIR}" ] && [ -f "${DIR}/backdrop.mvi" ] && [ -f "/usr/bin/showiframe" ]; then
			/usr/bin/showiframe ${DIR}/backdrop.mvi
			break
		fi
	done

	# Hook to execute scripts always before enigma2 start.
	if [ -f "/usr/bin/enigma2_pre_start.sh" ]; then
		/usr/bin/enigma2_pre_start.sh
	fi

	if [ "${BRANDCHECK}" == "dreambox" ]; then
		DISK=$(grep "/media/ba" /proc/mounts | awk '{gsub("/media/ba",""); print $1}' | head -n 1)
		umount -f /media/ba
		sleep 0.1
		umount -f "${DISK}"
		sleep 0.1
		mkdir -p /media/ba
		sleep 0.1
		mount "${DISK}" /media/ba
		sleep 0.1
	fi

	# Dreamci.
	if [ -f "/usr/bin/enigma2_pre_start_ciplus.sh" ]; then
		/usr/bin/enigma2_pre_start_ciplus.sh
	fi

	# Start enigma.
	sync

	(sleep 0.1; echo "Enigma2 is the main application so adjust oom score!"; PID=$(pidof enigma2); \
		[ -e "/proc/${PID}/oom_score_adj" ] && echo "-999" > "/proc/${PID}/oom_score_adj" || echo "-17" > "/proc/${PID}/oom_adj";) &

	# Set the debug level to be used for enigma2 logging.
	if [ -z "${ENIGMA_DEBUG_LVL}" ]; then
		DEBUGLEVEL=`sed -En 's|^config\.crash\.debugLevel=(.+)$|\1|p' "/etc/enigma2/settings"`
		[ -n "${DEBUGLEVEL}" ] || DEBUGLEVEL="3"
		DEBUG_LVL="${DEBUGLEVEL}"
	else
		DEBUG_LVL="${ENIGMA_DEBUG_LVL}"
	fi

	# Set time format used to prefix each line in the debug logs.
	DEBUGTIME=`sed -En 's|^config\.crash\.debugTimeFormat=(.+)$|\1|p' "/etc/enigma2/settings"`
	[ -n "${DEBUGTIME}" ] || DEBUGTIME="2"
	DEBUG_TIME="${DEBUGTIME}"

	if [ "${DEBUG_LVL}" -lt "4" ]; then
		LD_PRELOAD="${LIBS}" ENIGMA_DEBUG_LVL="${DEBUG_LVL}" ENIGMA_DEBUG_TIME="${DEBUG_TIME}" "/usr/bin/enigma2"
	else
		[ -n "${DEBUGDIR}" ] && [ -d "${DEBUGDIR}" ] || DEBUGDIR="/home/root/logs/"
		# Remove old logfiles.
		KEEP=`sed -En 's|^config\.crash\.debugFileCount=(.+)$|\1|p' "/etc/enigma2/settings"`
		[ -n "${KEEP}" ] || KEEP="5"
		for FILE in `ls -t "${DEBUGDIR}"enigma2_debug_*.log`; do
			let "KEEP--"
			if [ "${KEEP}" -lt "1" ]; then
				rm -f "${FILE}"
			fi
		done
		# Today's log file.
		FILE="${DEBUGDIR}enigma2_debug_$(date +%Y-%m-%d_%H-%M-%S).log"
		LD_PRELOAD="${LIBS}" ENIGMA_DEBUG_LVL="${DEBUG_LVL}" ENIGMA_DEBUG_TIME="${DEBUG_TIME}" "/usr/bin/enigma2" >> "${FILE}" 2>&1
	fi

	# Enigma2 exit codes:
	#
	#  1 - shutdown (halt)
	#  2 - reboot
	#  3 - restart enigma
	#  4 - front processor upgrade
	#  5 - install new settings
	#  6 - restart enigma in debug
	#  7 - manufacturer reset
	#  8 - reboot (force)
	# 12 - reboot to android
	# 16 - reboot to recovery
	# 42 - restart for unattended update
	# 43 - restart for autobackup restore
	# 44 - INI MICOM upgrade
	# 45 - GigaBlue WOL
	#
	# >128 signal

	RET=$?

	if [ "${RET}" -ne "1" ] && [ -f "/usr/bin/libreader" ]; then
		killall -9 libreader; sleep 5
	fi

	case ${RET} in
		1)
			if [ "${BRANDCHECK}" == "atto" ]; then
				echo "----" > /dev/player/panel
				echo -n "standby" > /sys/power/state
			fi
			if [ "${BRANDCHECK}" == "wetek" ]; then
				echo "0" > /sys/class/leds/wetek\:blue\:wifiled/brightness
				echo "0" > /sys/class/leds/wetek\:blue\:ethled/brightness
				echo "mem" > /sys/power/state
			fi
			if [ -f "/proc/stb/fp/led_color" ]; then
				echo "00ffffff" > /proc/stb/fp/led_color
			elif [ -f "/proc/stb/fp/led_brightness" ]; then
				echo "00000008" > /proc/stb/fp/led_brightness
			fi
			if [ -f "/usr/bin/turnoff_power" ]; then
				/usr/bin/turnoff_power
			elif [ -f "/usr/bin/hihalt" ]; then
				/usr/bin/hihalt
			else
				/sbin/halt
			fi
			;;
		2)
			[ -f "/proc/stb/fp/force_restart" ] && echo "1" > /proc/stb/fp/force_restart
			/sbin/reboot
			;;
		3)
			;;
		4)
			/sbin/rmmod lcd
			/usr/sbin/fpupgrade --upgrade 2>&1 | tee /home/root/fpupgrade.log
			sleep 0.1;
			/sbin/rmmod fp
			/sbin/modprobe fp
			/sbin/reboot
			;;
		5)
			if ! grep -q config.misc.RestartUI /etc/enigma2/settings; then
				echo "config.misc.RestartUI=true" >>/etc/enigma2/settings
			fi
			;;
		7)
			rm -fR /etc/enigma2
			;;
		8)
			/sbin/reboot -f
			;;
		12)
			if [ -e "/dev/block/by-name/bootoptions" ]; then
				mkdir -p /tmp/bootoptions
				mount /dev/block/by-name/bootoptions /tmp/bootoptions
				cp -f /tmp/bootoptions/STARTUP_ANDROID /tmp/bootoptions/STARTUP_ONCE
			else
				echo "rescue" > /proc/stb/fp/boot_mode
			fi
			[ -e "/usr/bin/setenv" ] && /usr/bin/setenv default_sys 1
			[ -e "/usr/sbin/fw_setenv" ] && /usr/sbin/fw_setenv bootfromnand 1
			/sbin/reboot
			;;
		16)
			if [ "${PLATFORMCHECK}" == "dmamlogic" ]; then
				/usr/sbin/to-the-rescue
			fi
			if [ -e "/dev/block/by-name/flag" ]; then
				echo -n "0" > /dev/block/by-name/flag
			elif [ -e "/dev/block/by-name/bootoptions" ]; then
				mkdir -p /tmp/bootoptions
				mount /dev/block/by-name/bootoptions /tmp/bootoptions
				cp -f /tmp/bootoptions/STARTUP_RECOVERY /tmp/bootoptions/STARTUP_ONCE
			else
				echo "rescue" > /proc/stb/fp/boot_mode
			fi
			/sbin/reboot
			;;
		42)
			df -P | grep -v "tmpfs " | awk '{print $6}' | tail -n +3 > /tmp/upgrade_mountpoints.txt
			while read LINE; do
				if [ -f "${LINE}/var/lib/opkg/status" ]; then
				DESTS="${DESTS}" --add-dest "${LINE}":"${LINE}"
			fi
			done < /tmp/upgrade_mountpoints.txt
			# Bind the console (when available).
			[ -f "/sys/class/vtconsole/vtcon1/bind" ] && echo "1" > /sys/class/vtconsole/vtcon1/bind
			PREVUPD="0"
			CURRUPD="999"
			opkg update 2>&1 | tee /home/root/opkgupgrade.log
			# update all other packages
			while [ "${CURRUPD}" -gt "0" ] && [ "${CURRUPD}" -ne "${PREVUPD}" ]; do
				opkg upgrade "${DESTS}" 2>&1 | tee -a /home/root/opkgupgrade.log
				PREVUPD="${CURRUPD}"
				CURRUPD=`opkg list-upgradable | wc -l`
				echo "===> ${CURRUPD} PACKAGE(S) REMAINING" >> /home/root/opkupgrade.log
			done
			/sbin/reboot
			;;
		43)
			# Auto install and autobackup.
			[ -f "/sys/class/vtconsole/vtcon1/bind" ] && echo "1" > /sys/class/vtconsole/vtcon1/bind
			/etc/init.d/settings-restore.sh
			if [ -f "/etc/init.d/avahi-daemon" ]; then
				/etc/init.d/avahi-daemon stop
			fi
			ifdown eth1
			ip addr flush dev eth1 scope global
			ifdown eth0
			ip addr flush dev eth0 scope global
			/etc/init.d/networking stop
			killall -9 udhcpc
			rm -f /var/run/udhcpc*
			/etc/init.d/dbus-1 reload
			/etc/init.d/networking start
			if [ -f "/etc/init.d/avahi-daemon" ]; then
				/etc/init.d/avahi-daemon start
			fi
			/etc/init.d/softcam restart
			break
			;;
		44)
			/usr/lib/enigma2/python/Plugins/SystemPlugins/MICOMUpgrade/bin/fbclear
			/usr/bin/showiframe /usr/lib/enigma2/python/Plugins/SystemPlugins/MICOMUpgrade/wait.mvi
			echo "fpupload" >/proc/vfd && sleep 3 && dd bs=256k if=/tmp/micom.bin of=/dev/mcu
			/usr/bin/showiframe /usr/lib/enigma2/python/Plugins/SystemPlugins/MICOMUpgrade/reboot.mvi
			# Wait forever for the user to power off.
			while(true) ; do sleep 60 ; done
			;;
		45)
			echo "    " > /dev/lcd0
			/usr/bin/gigablue_wol
			;;
		*)
			break
			;;
	esac
done
